/******************************************************************************
*
* project name:   TI-TimeWaste
* file name:      config.c
* initial date:   02/01/2001
* author:         thomas.nussbaumer@gmx.net
* description:    A Five-in-a-Row Solitaire Game
*
* $Id: config.c,v 1.2 2002/11/18 10:22:25 nussbaumer Exp $
*
******************************************************************************/

#define MAGIC_TIMEWASTE     "TIMEWASTE"
#define CFG_FNAME           "twcfg"
#define LEN_MAGIC_TIMEWASTE 6
#define MAX_LEVELS          20


short             nr_levels    = 0;
short             active_level = 0;
LEVEL_DEFINITION  levels[MAX_LEVELS] = {};

#define DEFAULT_NR_LEVELS 6
LEVEL_DEFINITION  default_levels[DEFAULT_NR_LEVELS]  = {
   {10,5,0,0,0,0L},
   {15,5,0,50,25,0L},
   {25,5,10,50,25,0L},
   {10,6,0,0,0,0L},
   {15,6,0,50,25,0L},
   {25,6,10,50,25,0L},
};


/*===========================================================================*/
/*                                                                           */
/*===========================================================================*/
short NextLine(FILE* fp,char* buffer,short size) {
    char* ret;

    do {
        if (!(ret = fgets(buffer,size,fp))) return 0;
        if (buffer[0] != '#') return 1;
    }
    while (1);
}


//=============================================================================
// very simple checksum routine to make cheating not too easy
//=============================================================================
unsigned short CalcChecksum(short idx) {
    unsigned short checksum = 0xaaaa;
    checksum -= (unsigned short)levels[idx].init_pieces<<2;
    checksum ^= ~(unsigned short)levels[idx].piece_types;
    checksum ^= (unsigned short)levels[idx].framed_percent;
    checksum ^= ~(unsigned short)levels[idx].price_bomb;
    checksum ^= (unsigned short)levels[idx].price_joker;
    checksum += ~(unsigned short)(levels[idx].hiscore & 0xffff);
    checksum ^= (unsigned short)(levels[idx].hiscore >> 16);
    return checksum;
}


//=============================================================================
// read levels - format:
//
// [init_pieces] [piece_types] [framed_percent] [price_bomb] [price_joker] [hiscore] [checksum]
//=============================================================================
short ParseLevelLine(char* buffer,short idx) {
    char* pos;

    if (!(pos = strtok(buffer," "))) return 0;
    levels[idx].init_pieces = atoi(pos);
    if (levels[idx].init_pieces>BOARD_SIZE) levels[idx].init_pieces = BOARD_SIZE;
    if (!(pos = strtok(NULL," "))) return 0;
    levels[idx].piece_types = atoi(pos);
    if (levels[idx].piece_types>NR_SPRITES) levels[idx].piece_types = NR_SPRITES;
    if (!(pos = strtok(NULL," "))) return 0;
    levels[idx].framed_percent = atoi(pos);
    if (levels[idx].framed_percent>100) levels[idx].framed_percent = 100;
    if (!(pos = strtok(NULL," "))) return 0;
    levels[idx].price_bomb = atoi(pos);
    if (levels[idx].price_bomb>999) levels[idx].price_bomb = 999;
    if (!(pos = strtok(NULL," "))) return 0;
    levels[idx].price_joker = atoi(pos);
    if (levels[idx].price_joker>999) levels[idx].price_joker = 999;
    if (!(pos = strtok(NULL," "))) return 0;
    levels[idx].hiscore = atol(pos);
    // no checksum !!
    if (!(pos = strtok(NULL," "))) levels[idx].hiscore=0;
    // invalid checksum !!
    else if (((unsigned short)atoi(pos)) != CalcChecksum(idx)) levels[idx].hiscore=0;
    return 1;
}


/*===========================================================================*/
/*                                                                           */
/*===========================================================================*/
short ReadConfigIntern() {
    char  buffer[256];
    short i;
    FILE* fp = fopen(CFG_FNAME,"r");

    if (!fp) return 1;

    //-------------------------------------------------------------------------
    // check magic marker
    //-------------------------------------------------------------------------
    if (!NextLine(fp,buffer,255)) {
        fclose(fp);
        //ST_showHelp("invalid marker1");ngetchx();
        return 0;
    }
    if (strncmp(buffer,MAGIC_TIMEWASTE,LEN_MAGIC_TIMEWASTE)) {
        fclose(fp);
        //ST_showHelp("invalid marker2");ngetchx();
        return 0;
    }

    //-------------------------------------------------------------------------
    // read number of levels
    //-------------------------------------------------------------------------
    if (!NextLine(fp,buffer,255)) {
        fclose(fp);
        //ST_showHelp("invalid nr_levels");ngetchx();
        return 0;
    }
    nr_levels = atoi(buffer);
    if (nr_levels > MAX_LEVELS) nr_levels = MAX_LEVELS;
    //-------------------------------------------------------------------------
    // read active level
    //-------------------------------------------------------------------------
    if (!NextLine(fp,buffer,255)) {
        fclose(fp);
        //ST_showHelp("invalid active_level");ngetchx();
        return 0;
    }
    active_level = atoi(buffer) - 1;
    if (active_level >= nr_levels) active_level = 0;

    for (i=0;i<nr_levels;i++) {
        if (!NextLine(fp,buffer,255)) {
            fclose(fp);
            //sprintf(buffer,"ERR1 L%d",i);ST_showHelp(buffer);ngetchx();
            return 0;
        }

        if (!ParseLevelLine(buffer,i)) {
            fclose(fp);
            //sprintf(buffer,"ERR2 L%d",i);ST_showHelp(buffer);ngetchx();
            return 0;
        }
    }

    fclose(fp);
    return 2;
}


/*===========================================================================*/
/*                                                                           */
/*===========================================================================*/
void ReadConfig() {
    short result = ReadConfigIntern();
    if (result != 2) {
        nr_levels = DEFAULT_NR_LEVELS;
        active_level = 0;
        memcpy(levels,default_levels,sizeof(LEVEL_DEFINITION)*DEFAULT_NR_LEVELS);
    }
    if (result == 0) {
        ST_showHelp("ERR: Invalid Cfg-File. Press a Key ...");
        ngetchx();
    }
}


/*===========================================================================*/
/*                                                                           */
/*===========================================================================*/
void WriteConfig() {
    short i;
    FILE* fp = fopen(CFG_FNAME,"w");

    if (!fp) return;

    fprintf(fp,"# [TI-FiaRS]\n# [format]\n# 1: init_pieces\n# 2: nr_types\n# 3: framed%%\n# 4: price bomb\n# 5: pricejoker\n# 6: hiscore\n# 7: checksum\n# [MAGIC_MARKER]\n");
    fprintf(fp,"%s\n",MAGIC_TIMEWASTE);
    fprintf(fp,"# [nr levels]\n%d\n",nr_levels);
    fprintf(fp,"# [active level]\n%d\n",active_level + 1);
    for (i=0;i<nr_levels;i++) {
        fprintf(fp,"# == level %d ==\n",i + 1);
        fprintf(fp,"%d %d %d %d %d %lu %04u\n",
                levels[i].init_pieces,levels[i].piece_types,
                levels[i].framed_percent,levels[i].price_bomb,
                levels[i].price_joker,levels[i].hiscore,CalcChecksum(i));
    }
    fclose(fp);
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: config.c,v $
// Revision 1.2  2002/11/18 10:22:25  nussbaumer
// generic commit
//
//
